baseVersion = '5.7.1'
cgcloudVersion = '1.6.0a1.dev393'
version = '5.7.1-b5cae9634820d76cb6c13b2a6312895122017d54'
distVersion = '5.7.1'
exactPython = 'python3.10'
python = 'python3.10'
dockerTag = '5.7.1-b5cae9634820d76cb6c13b2a6312895122017d54-py3.10'
currentCommit = 'b5cae9634820d76cb6c13b2a6312895122017d54'
dockerRegistry = 'quay.io/ucsc_cgl'
dockerName = 'toil'
dirty = False
cwltool_version = '3.1.20220628170238'



import functools
import warnings
from typing import Any, Callable, Union


def deprecated(new_function_name: str) -> Callable[..., Any]:
    def decorate(func: Callable[..., Any]) -> Callable[..., Any]:
        @functools.wraps(func)
        def call(*args: Any, **kwargs: Any) -> Any:
            warnings.warn(f'WARNING: "{func.__name__}()" is deprecated.  Please use "{new_function_name}()" instead.',
                          DeprecationWarning)
            return func(*args, **kwargs)
        return call
    return decorate


def compat_bytes(s: Union[bytes, str]) -> str:
    return s.decode('utf-8') if isinstance(s, bytes) else s



# Copyright (C) 2015-2022 Regents of the University of California
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import collections
import inspect
import logging
import os
import re
import socket
import threading
from functools import lru_cache
from urllib.request import urlopen
from urllib.error import URLError

from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    List,
    Optional,
    Tuple,
    TypeVar,
    Union,
    cast
)

import boto3
import boto3.resources.base
from boto3 import Session
import botocore
from botocore.credentials import JSONFileCache
from botocore.session import get_session
import boto.connection

logger = logging.getLogger(__name__)

@lru_cache(maxsize=None)
def establish_boto3_session(region_name: Optional[str] = None) -> Session:
    """
    This is the One True Place where Boto3 sessions should be established, and
    prepares them with the necessary credential caching.

    :param region_name: If given, the session will be associated with the given AWS region.
    """

    # Make sure to use credential caching when talking to Amazon via boto3
    # See https://github.com/boto/botocore/pull/1338/
    # And https://github.com/boto/botocore/commit/2dae76f52ae63db3304b5933730ea5efaaaf2bfc

    botocore_session = get_session()
    botocore_session.get_component('credential_provider').get_provider(
        'assume-role').cache = JSONFileCache()
    return Session(botocore_session=botocore_session, region_name=region_name)

@lru_cache(maxsize=None)
def client(service_name: str, *args: List[Any], region_name: Optional[str] = None, **kwargs: Dict[str, Any]) -> botocore.client.BaseClient:
    """
    Get a Boto 3 client for a particular AWS service.

    Global alternative to AWSConnectionManager.
    """
    session = establish_boto3_session(region_name=region_name)
    # MyPy can't understand our argument unpacking. See <https://github.com/vemel/mypy_boto3_builder/issues/121>
    client: botocore.client.BaseClient = session.client(service_name, *args, **kwargs) # type: ignore
    return client

@lru_cache(maxsize=None)
def resource(service_name: str, *args: List[Any], region_name: Optional[str] = None, **kwargs: Dict[str, Any]) -> boto3.resources.base.ServiceResource:
    """
    Get a Boto 3 resource for a particular AWS service.

    Global alternative to AWSConnectionManager.
    """
    session = establish_boto3_session(region_name=region_name)
    # MyPy can't understand our argument unpacking. See <https://github.com/vemel/mypy_boto3_builder/issues/121>
    resource: boto3.resources.base.ServiceResource = session.resource(service_name, *args, **kwargs) # type: ignore
    return resource

class AWSConnectionManager:
    """
    Class that represents a connection to AWS. Caches Boto 3 and Boto 2 objects
    by region.

    Access to any kind of item goes through the particular method for the thing
    you want (session, resource, service, Boto2 Context), and then you pass the
    region you want to work in, and possibly the type of thing you want, as arguments.

    This class is intended to eventually enable multi-region clusters, where
    connections to multiple regions may need to be managed in the same
    provisioner.

    Since connection objects may not be thread safe (see
    <https://boto3.amazonaws.com/v1/documentation/api/1.14.31/guide/session.html#multithreading-or-multiprocessing-with-sessions>),
    one is created for each thread that calls the relevant lookup method.
    """

    # TODO: mypy is going to have !!FUN!! with this API because the final type
    # we get out (and whether it has the right methods for where we want to use
    # it) depends on having the right string value for the service. We could
    # also individually wrap every service we use, but that seems like a good
    # way to generate a lot of boring code.

    def __init__(self) -> None:
        """
        Make a new empty AWSConnectionManager.
        """
        # This stores Boto3 sessions in .item of a thread-local storage, by
        # region.
        self.sessions_by_region: Dict[str, threading.local] = collections.defaultdict(threading.local)
        # This stores Boto3 resources in .item of a thread-local storage, by
        # (region, service name) tuples
        self.resource_cache: Dict[Tuple[str, str], threading.local] = collections.defaultdict(threading.local)
        # This stores Boto3 clients in .item of a thread-local storage, by
        # (region, service name) tuples
        self.client_cache: Dict[Tuple[str, str], threading.local] = collections.defaultdict(threading.local)
        # This stores Boto 2 connections in .item of a thread-local storage, by
        # (region, service name) tuples.
        self.boto2_cache: Dict[Tuple[str, str], threading.local] = collections.defaultdict(threading.local)

    def session(self, region: str) -> boto3.session.Session:
        """
        Get the Boto3 Session to use for the given region.
        """
        storage = self.sessions_by_region[region]
        if not hasattr(storage, 'item'):
            # This is the first time this thread wants to talk to this region
            # through this manager
            storage.item = establish_boto3_session(region_name=region)
        return cast(boto3.session.Session, storage.item)

    def resource(self, region: str, service_name: str) -> boto3.resources.base.ServiceResource:
        """
        Get the Boto3 Resource to use with the given service (like 'ec2') in the given region.
        """
        key = (region, service_name)
        storage = self.resource_cache[key]
        if not hasattr(storage, 'item'):
            # The Boto3 stubs are missing an overload for `resource` that takes
            # a non-literal string. See
            # <https://github.com/vemel/mypy_boto3_builder/issues/121#issuecomment-1011322636>
            storage.item = self.session(region).resource(service_name) # type: ignore
        return cast(boto3.resources.base.ServiceResource, storage.item)

    def client(self, region: str, service_name: str) -> botocore.client.BaseClient:
        """
        Get the Boto3 Client to use with the given service (like 'ec2') in the given region.
        """
        key = (region, service_name)
        storage = self.client_cache[key]
        if not hasattr(storage, 'item'):
            # The Boto3 stubs are probably missing an overload here too. See:
            # <https://github.com/vemel/mypy_boto3_builder/issues/121#issuecomment-1011322636>
            storage.item = self.session(region).client(service_name) # type: ignore
        return cast(botocore.client.BaseClient , storage.item)

    def boto2(self, region: str, service_name: str) -> boto.connection.AWSAuthConnection:
        """
        Get the connected boto2 connection for the given region and service.
        """
        if service_name == 'iam':
            # IAM connections are regionless
            region = 'universal'
        key = (region, service_name)
        storage = self.boto2_cache[key]
        if not hasattr(storage, 'item'):
            storage.item = getattr(boto, service_name).connect_to_region(region)
        return cast(boto.connection.AWSAuthConnection, storage.item)



import json
import logging
import os
import urllib.request
from typing import Dict, Optional

from botocore.client import BaseClient

from toil.lib.retry import retry

logger = logging.getLogger(__name__)


def get_flatcar_ami(ec2_client: BaseClient, architecture: str ='amd64') -> str:
    """
    Retrieve the flatcar AMI image to use as the base for all Toil autoscaling instances.

    AMI must be available to the user on AWS (attempting to launch will return a 403 otherwise).

    Priority is:
      1. User specified AMI via TOIL_AWS_AMI
      2. Official AMI from stable.release.flatcar-linux.net
      3. Search the AWS Marketplace

    If all of these sources fail, we raise an error to complain.

    :param ec2_client: Boto3 EC2 Client
    :param architecture: The architecture type for the new AWS machine. Can be either amd64 or arm64
    """
    # Take a user override
    ami = os.environ.get('TOIL_AWS_AMI')
    if not ami:
        logger.debug("No AMI found in TOIL_AWS_AMI; checking Flatcar release feed")
        ami = official_flatcar_ami_release(ec2_client=ec2_client, architecture=architecture)
    if not ami:
        logger.warning("No available AMI found in Flatcar release feed; checking marketplace")
        # TODO: Support architecture
        ami = aws_marketplace_flatcar_ami_search(ec2_client=ec2_client)
    if not ami:
        logger.critical("No available AMI found in marketplace")
        raise RuntimeError('Unable to fetch the latest flatcar image.')
    logger.info("Selected Flatcar AMI: %s", ami)
    return ami


@retry()  # TODO: What errors do we get for timeout, JSON parse failure, etc?
def official_flatcar_ami_release(ec2_client: BaseClient, architecture: str ='amd64') -> Optional[str]:
    """
    Check stable.release.flatcar-linux.net for the latest flatcar AMI.  Verify it's on AWS.

    :param ec2_client: Boto3 EC2 Client
    :param architecture: The architecture type for the new AWS machine. Can be either amd64 or arm64
    """
    # Flatcar images only live for 9 months.
    # Rather than hardcode a list of AMIs by region that will die, we use
    # their JSON feed of the current ones.

    JSON_FEED_URL = f'https://stable.release.flatcar-linux.net/{architecture}-usr/current/flatcar_production_ami_all.json'
    region = ec2_client._client_config.region_name  # type: ignore
    feed = json.loads(urllib.request.urlopen(JSON_FEED_URL).read())

    try:
        for ami_record in feed['amis']:
            # Scan the list of regions
            if ami_record['name'] == region:
                # When we find ours, return the AMI ID
                ami = ami_record['hvm']
                # verify it exists on AWS
                response = ec2_client.describe_images(Filters=[{'Name': 'image-id', 'Values': [ami]}])  # type: ignore
                if len(response['Images']) == 1 and response['Images'][0]['State'] == 'available':
                    return ami  # type: ignore
        # We didn't find it
        logger.warning(f'Flatcar image feed at {JSON_FEED_URL} does not have an image for region {region}')
    except KeyError:
        # We didn't see a field we need
        logger.warning(f'Flatcar image feed at {JSON_FEED_URL} does not have expected format')


@retry()  # TODO: What errors do we get for timeout, JSON parse failure, etc?
def aws_marketplace_flatcar_ami_search(ec2_client: BaseClient) -> Optional[str]:
    """Query AWS for all AMI names matching 'Flatcar-stable-*' and return the most recent one."""
    response: dict = ec2_client.describe_images(Owners=['aws-marketplace'],  # type: ignore
                                                Filters=[{'Name': 'name', 'Values': ['Flatcar-stable-*']}])
    latest: Dict[str, str] = {'CreationDate': '0lder than atoms.'}
    for image in response['Images']:
        if image["Architecture"] == "x86_64" and image["State"] == "available":
            if image['CreationDate'] > latest['CreationDate']:
                latest = image
    return latest.get('ImageId', None)



# Copyright (C) 2022 Regents of the University of California
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file contains WES utility functions borrowed from the Amazon Genomics
# CLI adapter codebase, at a6cc9904cd2d1899f7f64bf3c024a242c1a33749, to allow
# Toil to function as an AGC-compatible WES implementation without the use of a
# separate adapter. These functions help Toil use the WES semantics that AGC
# expects.

import json
import os
import sys
import zipfile
from os import path
from typing import Any, Dict, IO, List, Optional
if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict
from urllib.parse import urlparse, ParseResult

from toil.server.wes.abstract_backend import MalformedRequestException as InvalidRequestError

# These functions are licensed under the same Apache 2.0 license as Toil is,
# but they come from a repo with a NOTICE file, so we must preserve this notice
# text. We can print it if Toil ever grows a function to print third-party
# license and notice text. Note that despite what this notice says, the Apache
# license claims that it doesn't constitute part of the licensing terms.
NOTICE = """
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
"""

# The license also requires a note that these functions have been modified by us.
# They needed to be modified to typecheck.

# The official spec we are working with here is: https://aws.github.io/amazon-genomics-cli/docs/concepts/workflows/#multi-file-workflows

class WorkflowPlan(TypedDict):
    """
    These functions pass around dicts of a certain type, with `data` and `files` keys.
    """
    data: "DataDict"
    files: "FilesDict"

class DataDict(TypedDict, total=False):
    """
    Under `data`, there can be:
    * `workflowUrl` (required if no `workflowSource`): URL to main workflow code.
    """
    workflowUrl: str

class FilesDict(TypedDict, total=False):
    """
    Under `files`, there can be:
    * `workflowSource` (required if no `workflowUrl`): Open binary-mode file for the main workflow code.
    * `workflowInputFiles`: List of open binary-mode file for input files. Expected to be JSONs.
    * `workflowOptions`: Open binary-mode file for a JSON of options sent along with the workflow.
    * `workflowDependencies`: Open binary-mode file for the zip the workflow came in, if any.
    """
    workflowSource: IO[bytes]
    workflowInputFiles: List[IO[bytes]]
    workflowOptions: IO[bytes]
    workflowDependencies: IO[bytes]

def parse_workflow_zip_file(file: str, workflow_type: str) -> WorkflowPlan:
    """
    Processes a workflow zip bundle

    :param file: String or Path-like path to a workflow.zip file
    :param workflow_type: String, extension of workflow to expect (e.g. "wdl")

    :rtype: dict of `data` and `files`

    If the zip only contains a single file, that file is set as `workflowSource`

    If the zip contains multiple files with a MANIFEST.json file, the MANIFEST is used to determine
    appropriate `data` and `file` arguments. (See: parse_workflow_manifest_file())

    If the zip contains multiple files without a MANIFEST.json file:
      * a `main` workflow file with an extension matching the workflow_type is expected and will be set as `workflowSource`
      * optionally, if `inputs*.json` files are found in the root level of the zip, they will be set as `workflowInputs(_\d)*` in the order they are found
      * optionally, if an `options.json` file is found in the root level of the zip, it will be set as `workflowOptions`

    If the zip contains multiple files, the original zip is set as `workflowDependencies`
    """
    data: DataDict = dict()
    files: FilesDict = dict()

    wd = path.dirname(file)
    with zipfile.ZipFile(file) as zip:
        zip.extractall(wd)

        contents = zip.namelist()
        if not contents:
            raise RuntimeError("empty workflow.zip")

        if len(contents) == 1:
            # single file workflow
            files["workflowSource"] = open(path.join(wd, contents[0]), "rb")

        else:
            # multifile workflow
            if "MANIFEST.json" in contents:
                props = parse_workflow_manifest_file(path.join(wd, "MANIFEST.json"))

                if props.get("data"):
                    data.update(props["data"])

                if props.get("files"):
                    files.update(props["files"])

            else:
                if not f"main.{workflow_type.lower()}" in contents:
                    raise RuntimeError(f"'main.{workflow_type}' file not found")

                files["workflowSource"] = open(
                    path.join(wd, f"main.{workflow_type.lower()}"), "rb"
                )

                input_files = [f for f in contents if f.startswith("inputs")]
                if input_files:
                    if not files.get("workflowInputFiles"):
                        files["workflowInputFiles"] = []

                    for input_file in input_files:
                        files[f"workflowInputFiles"] += [
                            open(path.join(wd, input_file), "rb")
                        ]

                if "options.json" in contents:
                    files["workflowOptions"] = open(path.join(wd, "options.json"), "rb")

            # add the original zip bundle as a workflow dependencies file
            files["workflowDependencies"] = open(file, "rb")

    return {"data": data, "files": files}


def parse_workflow_manifest_file(manifest_file: str) -> WorkflowPlan:
    """
    Reads a MANIFEST.json file for a workflow zip bundle

    :param manifest_file: String or Path-like path to a MANIFEST.json file

    :rtype: dict of `data` and `files`

    MANIFEST.json is expected to be formatted like:
    .. code-block:: json
       {
           "mainWorkflowURL": "relpath/to/workflow",
           "inputFileURLs": [
               "relpath/to/input-file-1",
               "relpath/to/input-file-2",
               ...
           ],
           "optionsFileURL" "relpath/to/option-file
       }

    The `mainWorkflowURL` property that provides a relative file path in the zip to a workflow file, which will be set as `workflowSource`

    The inputFileURLs property is optional and provides a list of relative file paths in the zip to input.json files. The list is assumed
    to be in the order the inputs should be applied - e.g. higher list index is higher priority. If present, it will be used to set
    `workflowInputs(_\d)` arguments.

    The optionsFileURL property is optional and  provides a relative file path in the zip to an options.json file. If present, it will be
    used to set `workflowOptions`.

    """
    data: DataDict = dict()
    files: FilesDict = dict()
    with open(manifest_file, "rt") as f:
        manifest = json.loads(f.read())

    u = urlparse(manifest["mainWorkflowURL"])
    if not u.scheme or u.scheme == "file":
        # expect "/path/to/file" or "file:///path/to/file"
        # root is relative to the zip root
        files["workflowSource"] = open(
            workflow_manifest_url_to_path(u, path.dirname(manifest_file)), "rb"
        )

    else:
        data["workflowUrl"] = manifest["mainWorkflowUrl"]

    if manifest.get("inputFileURLs"):
        if not files.get("workflowInputFiles"):
            files["workflowInputFiles"] = []

        for url in manifest["inputFileURLs"]:
            u = urlparse(url)
            if not u.scheme or u.scheme == "file":
                files[f"workflowInputFiles"] += [
                    open(
                        workflow_manifest_url_to_path(u, path.dirname(manifest_file)),
                        "rb",
                    )
                ]

            else:
                raise InvalidRequestError(
                    f"unsupported input file url scheme for: '{url}'"
                )

    if manifest.get("optionsFileURL"):
        u = urlparse(manifest["optionsFileURL"])
        if not u.scheme or u.scheme == "file":
            files["workflowOptions"] = open(
                workflow_manifest_url_to_path(u, path.dirname(manifest_file)), "rb"
            )
        else:
            raise InvalidRequestError(
                f"unsupported option file url scheme for: '{manifest['optionFileURL']}'"
            )

    return {"data": data, "files": files}


def workflow_manifest_url_to_path(url: ParseResult, parent_dir: Optional[str] = None) -> str:
    """
    Interpret a possibly-relative parsed URL, relative to the given parent directory.
    """
    relpath = url.path if not url.path.startswith("/") else url.path[1:]
    if parent_dir:
        return path.join(parent_dir, relpath)
    return relpath



# Copyright (C) 2015-2021 Regents of the University of California
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import argparse
import logging
import os

from typing import Type

import connexion  # type: ignore

from toil.lib.misc import get_public_ip
from toil.server.wes.toil_backend import ToilBackend
from toil.server.wsgi_app import run_app
from toil.version import version
from toil.lib.aws import running_on_ec2, running_on_ecs, get_current_aws_region

logger = logging.getLogger(__name__)


def parser_with_server_options() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(description="Toil server mode.")

    parser.add_argument("--debug", action="store_true", default=False)
    parser.add_argument("--bypass_celery", action="store_true", default=False,
                        help="Skip sending workflows to Celery and just run them under the"
                             "server. For testing.")
    parser.add_argument("--host", type=str, default="127.0.0.1",
                        help="The host interface that the Toil server binds on. (default: '127.0.0.1').")
    parser.add_argument("--port", type=int, default=8080,
                        help="The port that the Toil server listens on. (default: 8080).")
    parser.add_argument("--swagger_ui", action="store_true", default=False,
                        help="If True, the swagger UI will be enabled and hosted on the "
                             "`{api_base_path}/ui` endpoint. (default: False)")
    # CORS
    parser.add_argument("--cors", action="store_true", default=False,
                        help="Enable Cross Origin Resource Sharing (CORS). This should only be turned on "
                             "if the server is intended to be used by a website or domain.")
    parser.add_argument("--cors_origins", type=str, default="*",
                        help="Ignored if --cors is False. This sets the allowed origins for CORS. "
                             "For details about CORS and its security risks, see: "
                             "https://w3id.org/ga4gh/product-approval-support/cors. (default: '*')")
    # production only
    parser.add_argument("-w", "--workers", dest='workers', type=int, default=2,
                        help="Ignored if --debug is True. The number of worker processes launched by the "
                             "WSGI server. (default: 2).")

    parser.add_argument("--work_dir", type=str, default=os.path.join(os.getcwd(), "workflows"),
                        help="The directory where workflows should be stored. This directory should be "
                             "empty or only contain previous workflows. (default: './workflows').")
    parser.add_argument("--state_store", type=str, default=None,
                        help="The local path or S3 URL where workflow state metadata should be stored. "
                             "(default: in --work_dir)")
    parser.add_argument("--opt", "-o", type=str, action="append", default=[],
                        help="Specify the default parameters to be sent to the workflow engine for each "
                             "run. Options taking arguments must use = syntax. Accepts multiple values.\n"
                             "Example: '--opt=--logLevel=CRITICAL --opt=--workDir=/tmp'.")
    parser.add_argument("--dest_bucket_base", type=str, default=None,
                        help="Direct CWL workflows to save output files to dynamically generated "
                             "unique paths under the given URL. Supports AWS S3.")
    parser.add_argument("--version", action='version', version=version)
    return parser


def create_app(args: argparse.Namespace) -> "connexion.FlaskApp":
    """
    Create a "connexion.FlaskApp" instance with Toil server configurations.
    """
    flask_app = connexion.FlaskApp(__name__,
                                   specification_dir='api_spec/',
                                   options={"swagger_ui": args.swagger_ui})

    flask_app.app.config['JSON_SORT_KEYS'] = False

    if args.cors:
        # enable cross origin resource sharing
        from flask_cors import CORS  # type: ignore
        CORS(flask_app.app, resources={r"/ga4gh/*": {"origins": args.cors_origins}})

    # add workflow execution service (WES) API endpoints
    backend = ToilBackend(work_dir=args.work_dir,
                          state_store=args.state_store,
                          options=args.opt,
                          dest_bucket_base=args.dest_bucket_base,
                          bypass_celery=args.bypass_celery)

    flask_app.add_api('workflow_execution_service.swagger.yaml',
                      resolver=connexion.Resolver(backend.resolve_operation_id))  # noqa

    # add custom endpoints
    if isinstance(backend, ToilBackend):
        # We extend the WES API to allow presenting log data
        base_url = "/toil/wes/v1"
        flask_app.app.add_url_rule(f"{base_url}/logs/<run_id>/stdout", view_func=backend.get_stdout)
        flask_app.app.add_url_rule(f"{base_url}/logs/<run_id>/stderr", view_func=backend.get_stderr)
        # To be a well-behaved AGC engine we can implement the default status check endpoint
        flask_app.app.add_url_rule("/engine/v1/status", view_func=backend.get_health)
        # And we can provide lost humans some information on what they are looking at
        flask_app.app.add_url_rule("/", view_func=backend.get_homepage)
        
    return flask_app


def start_server(args: argparse.Namespace) -> None:
    """ Start a Toil server."""

    # Explain a bit about who and where we are
    logger.info("Toil WES server version %s starting...", version)
    if running_on_ecs():
        logger.info("Environment appears to be Amazon ECS")
    if running_on_ec2():
        logger.info("Environment appears to be Amazon EC2")
    aws_region = get_current_aws_region()
    if aws_region:
        logger.info("AWS region appears to be: %s", aws_region)

    flask_app = create_app(args)

    host = args.host
    port = args.port

    if args.debug:
        flask_app.run(host=host, port=port)
    else:
        # start a production WSGI server
        run_app(flask_app.app, options={
            "bind": f"{host}:{port}",
            "workers": args.workers,
        })
